// =============================================================================
//   OpenGL1.5 - VCL Template (opengl15_vcl_template.zip)
// =============================================================================
//   Copyright  2003 by DGL - http://www.delphigl.com
// =============================================================================
//   Contents of this file are subject to the GNU Public License (GPL) which can
//   be obtained here : http://opensource.org/licenses/gpl-license.php
// =============================================================================
//   History :
//    Version 1.0 - Initial Release                            (Sascha Willems)
// =============================================================================
// BRON: http://dbfinteractive.com/index.php?topic=2117.15

unit OpenGL15_MainForm;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  dglOpenGL;

type
  TGLForm = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private-Deklarationen }
  public
    RC        : HGLRC;
    DC        : HDC;
    ShowFPS   : Boolean;
    FontBase  : GLUInt;
    StartTick : Cardinal;
    Frames    : Integer;
    FPS       : Single;
    procedure GoToFullScreen(pWidth, pHeight, pBPP, pFrequency : Word);
  end;

var
  GLForm: TGLForm;

const
 white: array[0..3] of single = ( 1.0, 1.0, 1.0, 1.0 );
 black: array[0..3] of single = ( 0.0, 0.0, 0.0, 1.0 ); //doubles as position of lights
 darklight: array[0..3] of single = ( -2.0, -2.0, -2.0, 2.0 );
 NLIGHTS: integer = 7;

var
 q: PGLUquadricObj;
 aotime: single;
 starttime: hresult;

implementation

{$R *.dfm}

uses mmsystem;

// =============================================================================
//  TForm1.GoToFullScreen
// =============================================================================
//  Wechselt in den mit den Parametern angegebenen Vollbildmodus
// =============================================================================
procedure TGLForm.GoToFullScreen(pWidth, pHeight, pBPP, pFrequency : Word);
var
 dmScreenSettings : DevMode;
begin
// Fenster vor Vollbild vorbereiten
WindowState := wsMaximized;
BorderStyle := bsNone;
ZeroMemory(@dmScreenSettings, SizeOf(dmScreenSettings));
with dmScreenSettings do
 begin
 dmSize              := SizeOf(dmScreenSettings);
 dmPelsWidth         := pWidth;                    // Breite
 dmPelsHeight        := pHeight;                   // Hhe
 dmBitsPerPel        := pBPP;                      // Farbtiefe
 dmDisplayFrequency  := pFrequency;                // Bildwiederholfrequenz
 dmFields            := DM_PELSWIDTH or DM_PELSHEIGHT or DM_BITSPERPEL or DM_DISPLAYFREQUENCY;
 end;
if (ChangeDisplaySettings(dmScreenSettings, CDS_FULLSCREEN) = DISP_CHANGE_FAILED) then
 begin
 MessageBox(0, 'Konnte Vollbildmodus nicht aktivieren!', 'Error', MB_OK or MB_ICONERROR);
 exit
 end;
end;



// =============================================================================
//  TForm1.FormCreate
// =============================================================================
//  OpenGL-Initialisierungen kommen hier rein
// =============================================================================
procedure TGLForm.FormCreate(Sender: TObject);
var
  i: integer;
begin
DecimalSeparator:='.'; //always use . as decimal seperator if it is , for dutch language!

// Wenn gewollt, dann hier in den Vollbildmodus wechseln
// Muss vorm Erstellen des Kontextes geschehen, da durch den Wechsel der
// Gertekontext ungltig wird!
// GoToFullscreen(1600, 1200, 32, 75);

// OpenGL-Funtionen initialisieren
InitOpenGL;
// Gertekontext holen
DC := GetDC(Handle);
// Renderkontext erstellen (32 Bit Farbtiefe, 24 Bit Tiefenpuffer, Doublebuffering)
RC := CreateRenderingContext(DC, [opDoubleBuffered], 32, 24, 0, 0, 0, 0);
// Erstellten Renderkontext aktivieren
ActivateRenderingContext(DC, RC);
// Tiefenpuffer aktivieren
glEnable(GL_DEPTH_TEST);
// Nur Fragmente mit niedrigerem Z-Wert (nher an Betrachter) "durchlassen"
glDepthFunc(GL_LESS);
// Lschfarbe fr Farbpuffer setzen
glClearColor(1.0,1.0,1.0,1.0);


//setup
  q := gluNewQuadric ();
   gluQuadricNormals (q,GLU_SMOOTH);

   glColorMaterial (GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
   glEnable(GL_COLOR_MATERIAL);
   // everything in the scene will receive white ambient light
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, @white);

  //setup all opengl lights to be "ambient occlusion" lights.
  for i:=0 to 8-1 do
  begin
	  glLightfv(GL_LIGHT0+i,GL_AMBIENT, @black);
    glLightfv(GL_LIGHT0+i,GL_SPECULAR, @black);
    // yo the secret sauce...
    // lights are set with NEGATIVE colour!
    glLightfv(GL_LIGHT0+i,GL_DIFFUSE, @darklight);
    // a quadratic falloff of light power is used
	  // attenuation is 1/(constant + linear*dist + quadratic*dist)
    glLightf(GL_LIGHT0+i, GL_CONSTANT_ATTENUATION,  1.0);//so intersecting spheres look right
    glLightf(GL_LIGHT0+i, GL_LINEAR_ATTENUATION,  0.0);
    glLightf(GL_LIGHT0+i, GL_QUADRATIC_ATTENUATION,  2.0); //physically correct would be 1.0, but it looks bad :-)
  end;
  // enable all sphere lights
  for i:=0 to NLIGHTS-1 do
  begin
    glEnable (GL_LIGHT0+i);
  end;
  // now the plane light needs differnt attenuation factors...
  glLightf(GL_LIGHT7, GL_CONSTANT_ATTENUATION,  0.0);//touch the plane and its black
  glLightf(GL_LIGHT7, GL_LINEAR_ATTENUATION,  1.0);
  glLightf(GL_LIGHT7, GL_QUADRATIC_ATTENUATION,  0.0);
  // enable plane light
  glEnable (GL_LIGHT7);

  glEnable(GL_LIGHTING);
  glEnable(GL_DEPTH_TEST);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(60.0,1.0,0.1,100.0);
  gluLookAt(0,0,-10.0, 0,0,0, 0,1,0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  aotime := 0;
  startTime := timeGetTime();
//end setup

// Idleevent fr Rendervorgang zuweisen
Application.OnIdle := ApplicationEventsIdle;
// Zeitpunkt des Programmstarts fr FPS-Messung speichern
StartTick := GetTickCount;
end;

procedure moveSpheres(val: integer);
begin
	glRotatef(aotime*0.4,1,1,1);
	glRotatef(val*aotime,val,5,3);
	glTranslatef(sin(val+aotime*0.1)*4.0,0,0);
end;

procedure moveGroundLight(val: integer);
begin
  glTranslatef(0,-6.0,0);
	glScalef(1.0,0.0,1.0);
	moveSpheres(val);
end;

procedure drawGround();
begin
   // draw GROUND plane
   glLoadIdentity();
   glTranslatef(0.0,-5.0,0.0);
   glRotatef (-90,1,0,0);
   glColor4f (1.0,1.0,0.9,1.0);
   // disable the ground light as it should not occlude itself
   glDisable (GL_LIGHT7);
   gluDisk(q,0.0,20.0,60,60);
   glEnable (GL_LIGHT7);
end;

procedure drawAOSpheres();
var
  i: integer;
begin
   // set the lights in position
   for i:=0 to NLIGHTS-1 do
   begin
	    glLoadIdentity();
		moveSpheres(i);
		// black is 0,0,0,1, which are the values we need for
		// a point light at the origin.
		glLightfv (GL_LIGHT0+i, GL_POSITION, @black);
   end;

   // draw the spheres being careful not to draw a sphere occluding itself
   for i:=0 to NLIGHTS-1 do
   begin
	// simulate a plane by always placing a light under the sphere on the plane
     glLoadIdentity();
	moveGroundLight(i);
	glLightfv (GL_LIGHT7, GL_POSITION, @black);
     // a sphere should not occlude itself so switch off its own light
	glDisable (GL_LIGHT0+i);
	glColor4f (1.0-(i/20.0),0.7+(i/20.0),1.0,1.0);
	glLoadIdentity();
	moveSpheres(i);
	gluSphere (q,1,30,30);
     glEnable (GL_LIGHT0+i);
   end;
end;

// =============================================================================
//  TForm1.FormDestroy
// =============================================================================
//  Hier sollte man wieder alles freigeben was man so im Speicher belegt hat
// =============================================================================
procedure TGLForm.FormDestroy(Sender: TObject);
begin
// Renderkontext deaktiveren
DeactivateRenderingContext;
// Renderkontext "befreien"
wglDeleteContext(RC);
// Erhaltenen Gertekontext auch wieder freigeben
ReleaseDC(Handle, DC);
// Falls wir im Vollbild sind, Bildschirmmodus wieder zurcksetzen
ChangeDisplaySettings(devmode(nil^), 0);

end;

// =============================================================================
//  TForm1.ApplicationEventsIdle
// =============================================================================
//  Hier wird gerendert. Der Idle-Event wird bei Done=False permanent aufgerufen
// =============================================================================
procedure TGLForm.ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
begin

glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

// Render the scene
aotime := (timeGetTime() - startTime) * 0.006;
	drawAOSpheres();
	drawGround();

// Show fps
//ShowText;

// Hinteren Puffer nach vorne bringen
SwapBuffers(DC);

// Windows denken lassen, das wir noch nicht fertig wren
Done := False;

// Nummer des gezeichneten Frames erhhen
inc(Frames);
// FPS aktualisieren
if GetTickCount - StartTick >= 500 then
 begin
 FPS       := Frames/(GetTickCount-StartTick)*1000;
 Frames    := 0;
 StartTick := GetTickCount
 end;
end;

// =============================================================================
//  TForm1.FormKeyPress
// =============================================================================
procedure TGLForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
case Key of
 #27 : Close;
end;
end;

end.
